%%
% *%%% Compute SS Decomposition and pF index for the optimal solutions from the ROOTCLUS model (Bocci & Vicari, 2019)
% April 2019
%
% Input:
% Load the mat file containing the optimal solution from the ROOTCLUS model
% ( you may either save the workspace as OptSol.mat file or modify the load command line appropriately)
%
% Output: 
% lambda; RSS/TSS; ESS/TSS; WSS/TSS; CWSS/TSS; IWSS/TSS; BSS/TSS; NP; JP; JM; RSSdf; ESSdf; pF
%

load 'OptSol' ...
    s Popt Mhopt Wopt Vhopt Costopt Fopt lambda offdiag;
  
nlambda=size(lambda,2);   
[N,J,H]=size(Mhopt(:,:,:,1));   
nodiag=1-eye(N);
if offdiag==1, s=s.*repmat(nodiag(:),1,H); end
TSS=ssq(s);

Decomp=zeros(13,nlambda); pF=zeros(1,nlambda);
Decomp(1,:)=lambda;

    for ll=1:nlambda
        P=Popt(:,:,ll);          % Optimal P matrix
        Mh=Mhopt(:,:,:,ll);      % Optimal Mh matrices
        W=Wopt(:,:,ll);          % Optimal W matrix
        Vh=Vhopt(:,:,ll);        % Optimal Vh matrix
        Cost=Costopt(:,:,ll);    % Optimal Constants
        FF=Fopt(ll);             % Optimal loss
    
    %% Compute Pseudo F from Decomposition
    oneJ=ones(J,1);  oneN2=ones(N^2,1);  if offdiag==1,oneN2=oneN2.*nodiag(:); end
    T=khatrirao(P,P); if offdiag==1, T=T.*repmat(nodiag(:),1,J);end
    w=diag(W);
    Qh=zeros(N^2,J,H); WSS=0; RSS=0;ESS=0;BSS=0;CWSS=0;IWSS=0;
    for h=1:H
        Qh(:,:,h)=khatrirao(Mh(:,:,h),P+Mh(:,:,h))+khatrirao(P,Mh(:,:,h));
        if offdiag==1, Qh(:,:,h)=Qh(:,:,h).*repmat(nodiag(:),1,J); end
        RSS=RSS+ssq(T*(w+Vh(h,:)')+Qh(:,:,h)*Vh(h,:)'+Cost(h)*oneN2);         
            Decomp(2,ll)=RSS/TSS;
        ESS=ESS+ssq(s(:,h)-T*(w+Vh(h,:)')-Qh(:,:,h)*Vh(h,:)'-Cost(h)*oneN2);  
            Decomp(3,ll)=ESS/TSS;
        WSS=WSS+ssq(T*(w+Vh(h,:)'+Cost(h)*oneJ)+Qh(:,:,h)*(Vh(h,:)'+Cost(h)*oneJ));  
            Decomp(4,ll)=WSS/TSS;     % Within SS
        BSS=BSS+ssq((oneN2-(T+Qh(:,:,h))*oneJ)*Cost(h));   
            Decomp(7,ll)=BSS/TSS;   % Between SS
        CWSS=CWSS+ssq(T*(w+Vh(h,:)'+Cost(h)*oneJ));        
            Decomp(5,ll)=CWSS/TSS;         % Common Within SS
        IWSS=IWSS+ssq(Qh(:,:,h)*(Vh(h,:)'+Cost(h)*oneJ));  
            Decomp(6,ll)=IWSS/TSS;         % Individual Within SS
        
    end
%      (IWSS/TSS)+(CWSS/TSS),WSS/TSS   
%      (BSS/TSS)+(WSS/TSS),RSS/TSS     
%      (RSS/TSS)+(ESS/TSS) 
%       Fopt, ESS/TSS,pause
    
        NP=sum(sum(P));  sP=sum(P);
        Decomp(8,ll)=NP;  % NP
    JP=sum(sum(P)>0);                                     % # of non-empty root clusters 
        Decomp(9,ll)=JP;  % JP
    JM=0; for h=1:H, JM=JM+sum(sum(Mh(:,:,h))>0); end     % total # of non-empty individual clusters
        Decomp(10,ll)=JM;  % JM
    RSSdf=NP+JP+JM+H+H*(N-NP);                            % RSS df
        Decomp(11,ll)=RSSdf;  % RSSdf
    
    if offdiag==0, ESSdf=(H*N*(N+1)/2)-RSSdf; elseif offdiag==1, ESSdf=(H*N*(N-1)/2)-RSSdf; end
        Decomp(12,ll)=ESSdf;  % ESSdf

     pF(1,ll)=(RSS/RSSdf)/(ESS/ESSdf); % pseudoF
        Decomp(13,ll)=pF(ll);  % pF
     
fprintf('lambda  %g, pF  %7.2f , %%RSS/TSS =  %5.2f , %%ESS/TSS =  %5.2f \n', lambda(ll), pF(ll), 100*RSS/TSS, 100*ESS/TSS)
        
     
    end

[maxpF,llmaxpF]=max(pF,[],2);  % position of lambda where pF is max

fprintf('\r max pF  %7.2f , lambda  %7.2f \n',maxpF,lambda(llmaxpF));
fprintf(' %%RSS/TSS= %5.2f , %%ESS/TSS= %5.2f \n %%WSS/TSS= %5.2f, %%CWSS/TSS= %5.2f, %%IWSS/TSS= %5.2f, %%BSS/TSS= %5.2f \n',100*Decomp(2,llmaxpF),...
    100*Decomp(3,llmaxpF),100*Decomp(4,llmaxpF),100*Decomp(5,llmaxpF),100*Decomp(6,llmaxpF),100*Decomp(7,llmaxpF));


%%%%%%%%%%%%%%%%%%%%%%
%%% NESTED FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%

%%%%%% Computing sum of squares
function t = ssq(a)
     t = sum(sum(a.^2));
end
%%%%%%

