% Program for ROOTCLUS (Bocci & Vicari, 2019)
% Written by L. Bocci and D. Vicari, Sapienza University of Rome, Italy (Last update: April, 2019)

% min f = min (F + l*G )
%
% where
% F = [sum(h) SS[ Sh - PWP' - (P+Mh) Vh (P+Mh)' - ch]] / sum(h) SS[ Sh ]
% G = ( N - NP ) / NP^2 
%
% and 
%
%   H, N             : number of subjects and objects
%   J                : number of clusters of objects
%   Sh               : (N x N) similarity matrix (h = 1,...,H)
%   P                : (N x J) incomplete membership binary matrix for objects with empty columns allowed (Root Partition)
%   NP               : number of objects in the Root  Partition 
%   W                : (J x J) non-negative diagonal weight matrix 
%   Mh               : (N x J) incomplete membership binary matrix for
%                      objects with empty columns allowed (h = 1,...,H)(Individual Partitions)
%   Vh               : (J x J) non-negative diagonal weight matrix (h = 1,...,H)
%   ch               : additive constant (h = 1,...,H)
%   l                : tuning parameter
%
% Input: 
%
%   S             : (N x N x H) matrix of Sh  
%   J             : number of clusters of objects (J?N) 
%   lambda        : (1 x nlambda) vector of l values
%   start     : starting points flag  ( 0 = random; 1 = current values for P,Mh,W,ch )  
%               If start=1, then starting matrices P0,Mh0,W0 and Cost0 =[(H x 1) vector of the constants] must be provided by the user
%   nstart    : number of random starts  
%   offdiag   : offdiagonal flag ( 0 = including diagonal entries; 1 = only offdiagonal entries ) 
%
% Output: optimal parameters in nstart replications
% 
%   Popt      : (N x J x nlambda) membership binary matrices across l values
%   Mhopt     : (N x J x H x nlambda) membership binary matrices across l values (h = 1,...,H)
%   Wopt      : (J x J x nlambda) non-negative weight matrices across l values  
%   Vhopt     : (H x J x nlambda) non-negative weight matrices across l values 
%               (the h-th row contains the main diagonal of the optimal Vh (h = 1,...,H))
%   Costopt   : (H x 1 x nlambda) vector of constants across l values
%   fopt      : (1 x nlambda) vector of the loss values across l values
%   iteropt   : (1 x nlambda) vector of the # of iterations across l values


warning off all;  optlsqnonneg=optimset('Display','off','TolFun',1e-6);

[~,N,H]=size(S); 
   
   %%%%%%%%%%%%%%%%%%%%% Preprocessing data when only offdiagonal entries (offdiag=1) 
   nodiag=1-eye(N);
   if offdiag==1, S=S.*repmat(nodiag,1,1,H); end
   %%%%%%%%%%%%%%%%%%%%%

   s=reshape(S,N^2,H);  
[nlambda]=size(lambda,2);
if start==1, nstart=1; end
fopt=zeros(1,nlambda);Fopt=zeros(1,nlambda);iteropt=zeros(1,nlambda);
Popt=zeros(N,J,nlambda); Wopt=zeros(J,J,nlambda); Vhopt=zeros(H,J,nlambda); 
Costopt=zeros(H,1,nlambda); Mhopt=zeros(N,J,H,nlambda);

for ll=1:nlambda
    fold=inf;
    for repl=1:nstart
        if mod(repl,10)==0, fprintf('ROOTCLUS: #Replication=%g, lambda  %g \n', repl, lambda(ll)), end

        %% ------ INITIALIZATION: STARTING VALUES P0,Mh0,W0,Cost0
        if start==0
            
            P0=zeros(N,J);
            [aP,~]=eigs(double(mean(S,3)),J);
            [~, waP]=max(abs(aP),[],2); 
            P0(:,1:max(waP))=(indic(waP));  
            ind=randperm(N);                   
            if J==N || J==N-1
                J0=N-2; %J0=1; 
            elseif J<N-1
                J0=J;
            end
                ind=ind(1:J0);
                P0(ind,:)=zeros(size(ind,2),J);
                whereP0=find(sum(P0)==0);              
                
                Mh0=zeros(N,J,H);
                for h=1:H
                    A=S(ind,ind,h); 
                    [aM,~]=eigs(double(A),size(ind,2));  
                    aM(:,whereP0)=zeros(J0,size(whereP0,2));    
                    [~, waM]=max(abs(aM),[],2); b=indic(waM); colb=size(b,2);
                    Mh0(ind,1:colb,h)=indic(waM);
                end
       
            T=khatrirao(P0,P0);
            if offdiag==1
                T=T.*repmat(nodiag(:),1,J);
            end
            W0=diag(lsqnonneg(T,mean(s,2),optlsqnonneg));  
   
            Cost0=zeros(H,1);
            
        end
        %% ------ END INITIALIZATION 
        
        [P,Mh,W,Vh,Cost,f,F,iter,Ldif]=ROOTCLUS_main(S,s,P0,Mh0,W0,Cost0,offdiag,lambda(ll));
 
         
        if (f<fold)                              
            fopt(ll)=f;
            Fopt(ll)=F;
            Popt(:,:,ll)=P;
            Mhopt(:,:,:,ll)=Mh;
            Wopt(:,:,ll)=W;
            Vhopt(:,:,ll)=Vh;
            Costopt(:,:,ll)=Cost;
            iteropt(ll)=iter;
            fold=f;
        end 
       
        if Ldif<-(1e-4)
            repl=repl-1; 
        end
    end        % endfor repl

    fprintf('ROOTCLUS: Optimal Loss Function after %g iters is: %12.8f \n\n',iteropt(ll),Fopt(ll));

end            % endfor lambda


disp('ROOTCLUS: Optimal Loss Function for lambda values'), [lambda; Fopt] 
disp('Optimal Parameter Estimates: Popt Wopt')
Popt,Wopt 
% disp('Optimal Parameter Estimates: Mhopt Vhopt Costopt')
% Mhopt, Vhopt, Costopt


%%%%%%%%%%%%%%%%%%%%%%
%%% NESTED FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%

%%%%%% Computing membership matrix
function [G] = indic (group)
[m,n] = size (group);
if m == 1
    m = n;
    n = 1;
    group = group (:);
end

if any (any (group - round(group))) ~= 0
    error ('each element of the input argument must be a positive integer');
end

maxg = max (group);
colstart = [0 cumsum(maxg)];
colstart (n + 1) = [];
colstart = reshape (colstart(ones(m,1),:),m*n,1);
colD = sum (maxg);
G = zeros (m, colD);
row = (1 : m)';
row = reshape (row(:,ones(n,1)),m*n,1);
idx = m*(colstart + group (:) - 1) + row;
G(idx) = ones (size(row));

end
%%%%%%
